unit hiLedDiamond;

interface

uses
  Kol,Windows,Share,Debug,Win;

type

  TStyle = (Raised, Lowered);
  TCheckMode = (mNone, mCheckBox, mRadioBtn);
  TPositionText = (tLeft,tRight,tCenter);

  THILedDiamond = class(THIWin)
  private
    FActive: Boolean;
    FActiveColor: TColor;
    FAutoInactiveColor: Boolean;
    FInactiveColor: TColor;
    FBackGroundColor: TColor;
    FLabel: PControl;
    function GetCenterPoint(Canvas: PCanvas): TPoint;
    procedure DrawBackGround(Canvas: PCanvas; BackGroundColor: TColor);
    procedure LedPaint(Canvas: PCanvas);
    procedure Paint(Sender: PControl; DC: HDC);
    procedure _OnClick(Sender: PObj);
    procedure SetValue(Value: boolean);
  public
    _prop_Data: TData;
    _prop_ColorOn: TColor;
    _prop_ColorOff: TColor;
    _prop_Value: boolean;
    _prop_AutoOffColor: boolean;
    _prop_CheckMode: byte;
    _prop_PosText : byte;
    _prop_Caption: string;
    _prop_Style: byte;
    _event_onClick: THI_Event;
    _event_onChange: THI_Event;

    procedure Init;
    destructor Destroy; override;
    property BackGroundColor: TCOlor read FBackGroundColor write FBackGroundColor;

    procedure _work_doOn(var _Data:TData; Index:word);
    procedure _work_doOff(var _Data:TData; Index:word);
    procedure _work_doChangeValue(var _Data:TData; Index:word);
    procedure _work_doColorOn(var _Data:TData; Index:word);
    procedure _work_doColorOff(var _Data:TData; Index:word);
    procedure _var_Value(var _Data:TData; Index:word);
    procedure _work_doCaption(var _Data: TData; Index: word);
    procedure _work_doValue(var _Data: TData; Index: word);
    procedure _work_doEnabled(var _Data: TData; Index: word);
    procedure _work_doVisible(var _Data: TData; Index: word);

  end;

implementation


procedure THILedDiamond._work_doVisible;
begin
  Control.Visible := ReadBool(_Data);
  if Assigned(FLabel) then
    FLabel.Visible := Control.Visible;
end;

procedure THILedDiamond._work_doEnabled;
begin
  Control.Enabled := ReadBool(_Data);
  Control.Invalidate;
end;

procedure THILedDiamond._work_doCaption;
begin
  if Assigned(FLabel) then
    FLabel.Caption := ToString(_Data);
end;

procedure THILedDiamond.SetValue(Value: boolean);
var
  i: integer;
begin
  if TCheckMode(_prop_CheckMode) = mCheckBox then
  begin
    FActive := Value;
    Control.Invalidate;
    _hi_onEvent(_event_onChange, Integer(FActive));
  end;

  if TCheckMode(_prop_CheckMode) = mRadioBtn then
  begin
    for i := 0 to FParent.ChildCount-1 do
      if FParent.Children[i].SubClassName = 'THILedDiamond' then
      begin
        SetProp(FParent.Children[i].Handle,PChar('LOCK'),0);
        FParent.Children[i].Invalidate;
      end;
    SetProp(Control.Handle,PChar('LOCK'),1);
    FActive := True;
    Control.Invalidate;
    _hi_onEvent(_event_onChange, Integer(FActive));
  end;

  if TCheckMode(_prop_CheckMode) = mNone then
  begin
    Control.Invalidate;
    _hi_onEvent(_event_onChange, Integer(FActive));
  end;

end;

procedure THILedDiamond._work_doValue;
begin
  if TCheckMode(_prop_CheckMode) = mNone then
  begin
    FActive := ReadBool(_Data);
    Control.Invalidate;
    _hi_onEvent(_event_onChange,Integer(FActive));
    Exit;
  end;

  if TCheckMode(_prop_CheckMode) = mCheckBox then
  begin
    FActive := ReadBool(_Data);
    SetValue(FActive);
    Exit;
  end;
  //mRadioBtn
  if ReadBool(_Data) then SetValue(True);
end;

procedure THILedDiamond._OnClick(Sender: PObj);
begin
  if TCheckMode(_prop_CheckMode) <> mNone then
    SetValue(not FActive);

  _hi_onEvent(_event_onClick,_prop_Data);
end;

procedure THILedDiamond._work_doOn;
begin
  if TCheckMode(_prop_CheckMode) = mNone then
  begin
    FActive := True;
    Control.Invalidate;
    _hi_onEvent(_event_onChange,Integer(FActive));
  end else SetValue(True);

end;

procedure THILedDiamond._work_doOff;
begin
  if TCheckMode(_prop_CheckMode) = mRadioBtn then Exit;
  if TCheckMode(_prop_CheckMode) = mNone then
  begin
    FActive := False;
    Control.Invalidate;
    _hi_onEvent(_event_onChange,Integer(FActive));
  end else SetValue(False);
end;

procedure THILedDiamond._work_doChangeValue;
begin
  if TCheckMode(_prop_CheckMode) = mRadioBtn then Exit;
  if TCheckMode(_prop_CheckMode) = mNone then
  begin
    FActive := not FActive;
    Control.Invalidate;
    _hi_onEvent(_event_onChange,Integer(FActive));
  end else SetValue(not FActive);
end;

procedure THILedDiamond._work_doColorOn;
begin
  FActiveColor := ToInteger(_Data);
  Control.Invalidate;
end;

procedure THILedDiamond._work_doColorOff;
begin
  FInactiveColor := ToInteger(_Data);
  Control.Invalidate;
end;

procedure THILedDiamond._var_Value;
begin
  dtInteger(_Data,Integer(FActive));
end;

procedure THILedDiamond.Init;
begin
  Control := NewPaintBox(FParent);
  inherited;
  Control.SubClassName := 'THILedDiamond';
  Control.DoubleBuffered := True;
  FActiveColor := _prop_ColorOn;
  FInactiveColor := _prop_ColorOff;
  FActive := _prop_Value;
  FBackGroundColor := FParent.Color;
  FAutoInactiveColor := _prop_AutoOffColor;
  if (TCheckMode(_prop_CheckMode) = mRadioBtn) then
    SetProp(Control.Handle,PChar('LOCK'),Integer(FActive));

  //if _prop_Caption <> '' then
  //begin
    if TPositionText(_prop_PosText) = tCenter then
      Flabel := Newlabel(FParent,_prop_Caption)
    else
      Flabel := Newlabel(Control,_prop_Caption);
    Flabel.Font.Assign(Control.Font);
    Flabel.AutoSize(True);
    Flabel.CursorLoad(0, MakeIntResource(_prop_Cursor));
    Flabel.Show;
    Flabel.OnClick := _OnClick;

    case TPositionText(_prop_PosText) of
     tRight:
      begin
        Flabel.Top := 0;
        Flabel.Height := Control.Height;
        Flabel.Left := Control.Height + 6;
        Flabel.VerticalAlign := vaCenter;
      end;
     tLeft:
      begin
        Flabel.TextAlign := taRight;
        Flabel.VerticalAlign := vaCenter;
        Flabel.Top := 0;
        Flabel.Height := Control.Height;
        Flabel.Left := Control.Width - Control.Height - 6 - Flabel.Width;
      end;
     tCenter:
      begin
        Flabel.TextAlign := taCenter;
        Flabel.Top := Control.Top + Control.Height+2;
        Flabel.Left := (Control.Left + (Control.Width div 2))  - (Flabel.Width div 2);
      end;
    end;

  Control.OnClick := _OnClick;
  Control.Onpaint := Paint;
  Control.Invalidate;

  Flabel.Invalidate;
end;

destructor THILedDiamond.Destroy;
begin

  inherited;
end;

procedure THILedDiamond.Paint(Sender: PControl; DC: HDC);
begin
  if (TCheckMode(_prop_CheckMode) = mRadioBtn) then
    FActive := Boolean(GetProp(Control.Handle,PChar('LOCK')));

  LedPaint(Control.Canvas);
  Flabel.Invalidate;
end;

function THILedDiamond.GetCenterPoint(Canvas: PCanvas): TPoint;
begin
    case TPositionText(_prop_PosText) of
     tRight:
      begin
        Result := MakePoint((Control.Height) div 2, (Control.Height) div 2);
      end;
     tLeft:
      begin
        Result := MakePoint(Control.Width-(Control.Height div 2) , (Control.Height) div 2);
      end;
     tCenter:
      begin
        Result := MakePoint((Control.Width) div 2 , (Control.Height) div 2);
      end;
    end;
end;

function DimColor(Value: TColor; Divisor: Integer): TColor;
var
  Red, Green, Blue: Integer;
begin
  if Divisor > 0 then
  begin
    Red := ((Value and $FF)) div Divisor;
    Green := ((Value and $FF00) shr 8) div Divisor;
    Blue := ((Value and $FF0000) shr 16) div Divisor;
  end
  else
  begin
    Red := (Value and $FF) * -Divisor;
    Green := ((Value and $FF00) shr 8) * -Divisor;
    Blue := ((Value and $FF0000) shr 16) * -Divisor;
  end;

  if Red > 255 then Red := 255;
  if Green > 255 then Green := 255;
  if Blue > 255 then Blue := 255;

  if Red < 0 then Red := 0;
  if Green < 0 then Green := 0;
  if Blue < 0 then Blue := 0;

  Result := Red + (Green shl 8) + (Blue shl 16);
end;

procedure THILedDiamond.LedPaint(Canvas: PCanvas);
var
  DrawRect: TRect;
  Point1: TPoint;
  Point2: TPoint;
  Point3: TPoint;
  Point4: TPoint;
  CenterPoint: TPoint;
  HalfSize: Integer;
begin
  with Canvas{$IFNDEF F_P}^{$ENDIF}, DrawRect do
  begin
    DrawBackGround(Canvas, BackGroundColor);


    DrawRect := MakeRect(Control.Width-Control.Height, 0, Control.Width, Control.Height);


    CenterPoint := GetCenterPoint(Canvas);
    HalfSize := (Control.Height - 1) div 2;

    if FActive then
    begin
      Canvas.Brush.Color := FActiveColor;
      Canvas.Pen.Color := FActiveColor;
    end
    else
    begin
      if FAutoInactiveColor then
      begin
        Canvas.Brush.Color := DimColor(FActiveColor, 3);
        Canvas.Pen.Color := Brush.Color;
      end
      else
      begin
        Canvas.Brush.Color := FInactiveColor;
        Canvas.Pen.Color := Brush.Color;
      end
    end;

    if not Control.Enabled then
    begin
      Canvas.Brush.Color := clInactiveCaption;
      if Assigned(FLabel) then
        FLabel.Enabled := False;
    end else
    if Assigned(FLabel) then
      FLabel.Enabled := True;

    Point1 := MakePoint(CenterPoint.X, CenterPoint.Y - HalfSize);
    Point2 := MakePoint(CenterPoint.X + HalfSize, CenterPoint.Y);
    Point3 := MakePoint(CenterPoint.X, CenterPoint.Y + HalfSize);
    Point4 := MakePoint(CenterPoint.X - HalfSize, CenterPoint.Y);

    Polygon([Point1, Point2, Point3, Point4]);

    case TStyle(_prop_Style) of
      Lowered: begin
          Pen.Color := clBlack;
          Polygon([Point1, Point4]);

          Pen.Color := clBtnShadow;
          Polygon([Point1, Point2]);
          Polygon([Point3, Point4]);

          Pen.Color := clBtnHighlight;
          Polygon([Point2, Point3]);
        end;
      Raised: begin
          Pen.Color := clBlack;
          Polygon([Point2, Point3]);
          Pen.Color := clBtnHighlight;
          Polygon([Point1, Point4]);

          Pen.Color := clBtnShadow;
          Polygon([Point1, Point2]);
          Polygon([Point3, Point4]);
        end;
    end;
  end;
end;

procedure THILedDiamond.DrawBackGround(Canvas: PCanvas; BackGroundColor: TColor);
var
  DrawRect: TRect;
begin
  DrawRect := MakeRect(0, 0, Control.Width, Control.Height);
  Canvas.Brush.BrushStyle := bsSolid;
  Canvas.Brush.Color := BackGroundColor;
  Canvas.FillRect(MakeRect(0, 0, Control.Width, Control.Height));
end;



end.
